@@ -2,7 +2,7 @@
 from functools import reduce
 from pprint import pprint
 
-from . import avo_ast
+from . import expressions, statements
 from .combinator import *
 from .types import Token, TokenType
 
@@ -29,8 +29,8 @@ id_tag = Tag(TokenType.ID)
 num = Tag(TokenType.INT) ^ (lambda x: int(x))
 float_num = Tag(TokenType.FLOAT) ^ (lambda x: float(x))
 boolean = Tag(TokenType.BOOL) ^ process_boolean
-null = keyword('null') ^ (lambda x: avo_ast.NullAST())
-string = Tag(TokenType.STRING) ^ (lambda x: avo_ast.StringAST(x[1:-1]))
+null = keyword('null') ^ (lambda x: expressions.NullAST())
+string = Tag(TokenType.STRING) ^ (lambda x: expressions.StringAST(x[1:-1]))
 a_expr_precedence_levels = [
     ['*', '/'],
     ['+', '-', '%'],
@@ -51,7 +51,7 @@ def array_expr():
     """
     def process(p):
         (_, data), _ = p
-        return avo_ast.ArrayAST(data)
+        return expressions.ArrayAST(data)
     return keyword('[') + Opt(Rep(Lazy(expr) + Opt(keyword(',')))) + keyword(']') ^ process
 
 
@@ -63,7 +63,7 @@ def array_generator_expr():
         ((((((_, var), _), val), _), obj), condition), _ = p
         if condition is not None:
             _, condition = condition
-        return avo_ast.GeneratorAST(var, val, obj, condition)
+        return expressions.GeneratorAST(var, val, obj, condition)
     return (
             keyword('[') + expression() + keyword('for') + id_tag + operator('in') + expression() +
             Opt(keyword('if') + Exp(b_expr(), None)) + keyword(']')
@@ -77,7 +77,7 @@ def if_else_expr():
     """
     def process(p):
         (((body, op1), condition), op2), else_body = p
-        return avo_ast.TernaryOpAST(body, op1, condition, op2, else_body)
+        return expressions.TernaryOpAST(body, op1, condition, op2, else_body)
     return (
             Lazy(expr) + Alt(keyword('if'), operator('?')) + Lazy(expr) +
             Alt(keyword('else'), operator(':')) + Lazy(expression)
@@ -89,14 +89,14 @@ def lambda_stmt():
     (a, b, c) => {...}
     """
     def process(p):
-        (((((_, args), _), _), _), statements), _ = p
+        (((((_, args), _), _), _), stmts), _ = p
         arguments = []
         for arg in args:
             if arg.value[0][1] is None:
-                arguments.append(avo_ast.ArgumentAST(arg.value[0][0], None))
+                arguments.append(expressions.ArgumentAST(arg.value[0][0], None))
             else:
-                arguments.append(avo_ast.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
-        return avo_ast.LambdaStmt(arguments, statements)
+                arguments.append(expressions.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
+        return statements.LambdaStmt(arguments, stmts)
     return (
             keyword('(') + Rep(id_tag + Opt(operator('=') + Lazy(expression)) + Opt(keyword(','))) +
             keyword(')') + operator('=>') + keyword('{') + Opt(Lazy(stmt_list)) + keyword('}')
@@ -109,7 +109,7 @@ def module_obj_expr():
     """
     def process(p):
         (module, _), var = p
-        return avo_ast.ModuleCallAST(module, var)
+        return expressions.ModuleCallAST(module, var)
     return id_tag + operator('.') + id_tag ^ process
 
 
@@ -126,10 +126,10 @@ def a_expr_value():
             brace_expr() |
             module_obj_expr() |
             Lazy(class_property_expr) |
-            (num ^ (lambda x: avo_ast.IntAST(x))) |
-            (float_num ^ (lambda x: avo_ast.FloatAST(x))) |
-            (id_tag ^ (lambda x: avo_ast.VarAST(x))) |
-            (boolean ^ (lambda x: avo_ast.BoolAST(x))) |
+            (num ^ (lambda x: expressions.IntAST(x))) |
+            (float_num ^ (lambda x: expressions.FloatAST(x))) |
+            (id_tag ^ (lambda x: expressions.VarAST(x))) |
+            (boolean ^ (lambda x: expressions.BoolAST(x))) |
             string |
             null
     )
@@ -153,7 +153,7 @@ def a_expr_term():
 
 
 def process_binop(op):
-    return lambda l, r: avo_ast.BinOpAST(op, l, r)
+    return lambda l, r: expressions.BinOpAST(op, l, r)
 
 
 def any_op_in_list(ops):
@@ -177,7 +177,7 @@ def a_expr():
 # --== Boolean conditions ==-- #
 def process_relop(p):
     (l, op), r = p
-    return avo_ast.RelativeOp(op, l, r)
+    return expressions.RelativeOp(op, l, r)
 
 
 def b_expr_relop():
@@ -197,7 +197,7 @@ def b_expr_not():
     not x
     !x
     """
-    return (Alt(keyword('not'), operator('!')) + Lazy(b_expr_term)) ^ (lambda p: avo_ast.NotOp(p[1]))
+    return (Alt(keyword('not'), operator('!')) + Lazy(b_expr_term)) ^ (lambda p: expressions.NotOp(p[1]))
 
 
 def b_expr_group():
@@ -205,17 +205,17 @@ def b_expr_group():
 
 
 def b_expr_term():
-    return b_expr_group() | b_expr_not() | b_expr_relop() | (boolean ^ (lambda x: avo_ast.BoolAST(x)))
+    return b_expr_group() | b_expr_not() | b_expr_relop() | (boolean ^ (lambda x: expressions.BoolAST(x)))
 
 
 def process_logic(op):
     match op:
         case 'and' | '&&':
-            return lambda l, r: avo_ast.AndOp(l, r)
+            return lambda l, r: expressions.AndOp(l, r)
         case 'or' | '||':
-            return lambda l, r: avo_ast.OrOp(l, r)
+            return lambda l, r: expressions.OrOp(l, r)
         case 'in':
-            return lambda l, r: avo_ast.InOp(l, r)
+            return lambda l, r: expressions.InOp(l, r)
         case _:
             raise RuntimeError(f'unknown logic operator: {op}')
 
@@ -234,7 +234,7 @@ def brace_expr():
         for i in v_arr:
             (_, i), _ = i.value
             arr.append(i)
-        return avo_ast.BraceAST(obj, [v] + arr)
+        return expressions.BraceAST(obj, [v] + arr)
     return (
             (Lazy(array_expr) | Lazy(call_stmt) | string | id_or_module()) + keyword('[') +
             Lazy(expr) + keyword(']') + Rep(keyword('[') + Lazy(expr) + keyword(']'))
@@ -256,7 +256,7 @@ def class_property_expr():
         ((is_super, name), _), var = p
         if is_super is not None:
             is_super = True
-        return avo_ast.ClassPropAST(name, var, is_super)
+        return expressions.ClassPropAST(name, var, is_super)
     return Opt(keyword('super')) + Alt(id_tag, keyword('this')) + operator('::') + id_tag ^ process
 
 
@@ -271,7 +271,7 @@ def assign_stmt():
     """
     def process(p):
         ((_, name), _), e = p
-        return avo_ast.AssignStmt(name, e, False, True)
+        return statements.AssignStmt(name, e, False, True)
     return (
             keyword('var') + id_tag + operator('=') + expression()
     ) ^ process
@@ -283,7 +283,7 @@ def assign_const_stmt():
     """
     def process(p):
         ((_, name), _), e = p
-        return avo_ast.AssignStmt(name, e, True, True)
+        return statements.AssignStmt(name, e, True, True)
     return (keyword('const') + id_tag + operator('=') + expression()) ^ process
 
 
@@ -293,7 +293,7 @@ def reassign_stmt():
     """
     def process(p):
         (name, op), e = p
-        return avo_ast.AssignStmt(name, e, False, False, op)
+        return statements.AssignStmt(name, e, False, False, op)
     return ((brace_expr() | id_or_module()) + any_op_in_list(assign_operators) + expression()) ^ process
 
 
@@ -306,13 +306,15 @@ def unary_op_stmt():
         sym, name = p
         if sym not in unary_operators:
             name, sym = sym, name
-        return avo_ast.UnaryOpAST(sym, avo_ast.VarAST(name))
-    return Alt(any_op_in_list(unary_operators) + id_or_module(), id_or_module() + any_op_in_list(unary_operators)) ^ process
+        return expressions.UnaryOpAST(sym, expressions.VarAST(name))
+    return Alt(
+        any_op_in_list(unary_operators) + id_or_module(), id_or_module() + any_op_in_list(unary_operators)
+    ) ^ process
 
 
 def stmt_list():
     def process(rep):
-        return avo_ast.StmtList(rep)
+        return statements.StmtList(rep)
     return Rep(Lazy(stmt) + Opt(keyword(';')) ^ (lambda x: x[0])) ^ process
 
 
@@ -343,7 +345,7 @@ def if_stmt():
             false_body = None
         if elif_array:
             elif_array = [i.value for i in elif_array]
-        return avo_ast.IfStmt(condition, body, elif_array, false_body)
+        return statements.IfStmt(condition, body, elif_array, false_body)
     result = keyword('if') + Exp(b_expr(), None) + keyword('{') + Opt(Lazy(stmt_list)) + keyword('}')
     result += Opt(
         Rep(
@@ -364,7 +366,7 @@ def while_stmt():
     """
     def process(p):
         (((_, condition), _), body), _ = p
-        return avo_ast.WhileStmt(condition, body)
+        return statements.WhileStmt(condition, body)
     result = keyword('while') + Exp(b_expr(), None) + keyword('{') + Opt(Lazy(stmt_list)) + keyword('}')
     return result ^ process
 
@@ -373,14 +375,14 @@ def break_stmt():
     """Break statement
     break
     """
-    return keyword('break') ^ (lambda x: avo_ast.BreakStmt())
+    return keyword('break') ^ (lambda x: statements.BreakStmt())
 
 
 def continue_stmt():
     """Continue statement
     continue
     """
-    return keyword('continue') ^ (lambda x: avo_ast.ContinueStmt())
+    return keyword('continue') ^ (lambda x: statements.ContinueStmt())
 
 
 def echo_stmt():
@@ -389,7 +391,7 @@ def echo_stmt():
     """
     def process(p):
         (_, data), _ = p
-        return avo_ast.EchoStmt(data)
+        return statements.EchoStmt(data)
     return (
             keyword('echo') + keyword('(') +
             Opt(
@@ -406,7 +408,7 @@ def read_stmt():
     """
     def process(p):
         _, text = p
-        return avo_ast.ReadStmt(text)
+        return statements.ReadStmt(text)
     return keyword('read') + expression() ^ process
 
 
@@ -417,14 +419,14 @@ def func_stmt():
     }
     """
     def process(p):
-        ((((((_, func_name), _), args), _), _), statements), _ = p
+        ((((((_, func_name), _), args), _), _), stmts), _ = p
         arguments = []
         for arg in args:
             if arg.value[0][1] is None:
-                arguments.append(avo_ast.ArgumentAST(arg.value[0][0], None))
+                arguments.append(expressions.ArgumentAST(arg.value[0][0], None))
             else:
-                arguments.append(avo_ast.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
-        return avo_ast.FuncStmt(func_name, arguments, statements)
+                arguments.append(expressions.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
+        return statements.FuncStmt(func_name, arguments, stmts)
     return (
             keyword('func') + id_tag + keyword('(') +
             Rep(id_tag + Opt(operator('=') + Lazy(expression)) + Opt(keyword(','))) +
@@ -441,10 +443,10 @@ def interface_func_stmt():
         arguments = []
         for arg in args:
             if arg.value[0][1] is None:
-                arguments.append(avo_ast.ArgumentAST(arg.value[0][0], None))
+                arguments.append(expressions.ArgumentAST(arg.value[0][0], None))
             else:
-                arguments.append(avo_ast.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
-        return avo_ast.FuncStmt(func_name, arguments, avo_ast.StmtList([]))
+                arguments.append(expressions.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
+        return statements.FuncStmt(func_name, arguments, statements.StmtList([]))
     return (
             keyword('func') + id_tag + keyword('(') +
             Rep(id_tag + Opt(operator('=') + Lazy(expression)) + Opt(keyword(','))) +
@@ -460,16 +462,16 @@ def call_stmt():
     }
     """
     def process(p):
-        (((func_name, _), args), _), l = p
+        (((func_name, _), args), _), lambda_body = p
         arguments = []
         for arg in args:
             if arg.value[0][0] is None:
-                arguments.append(avo_ast.ArgumentAST(None, arg.value[0][1]))
+                arguments.append(expressions.ArgumentAST(None, arg.value[0][1]))
             else:
-                arguments.append(avo_ast.ArgumentAST(arg.value[0][0][0], arg.value[0][1]))
-        if l:
-            arguments.append(avo_ast.ArgumentAST(None, avo_ast.LambdaStmt([], l[0][1])))
-        return avo_ast.CallStmt(func_name, arguments)
+                arguments.append(expressions.ArgumentAST(arg.value[0][0][0], arg.value[0][1]))
+        if lambda_body:
+            arguments.append(expressions.ArgumentAST(None, statements.LambdaStmt([], lambda_body[0][1])))
+        return statements.CallStmt(func_name, arguments)
     return (
         id_or_module() + keyword('(') +
         Rep(Opt(id_tag + operator('=')) + expression() + Opt(keyword(','))) +
@@ -483,7 +485,7 @@ def return_stmt():
     """
     def process(p):
         _, return_value = p
-        return avo_ast.ReturnStmt(return_value)
+        return statements.ReturnStmt(return_value)
     return keyword('return') + Opt(expression()) ^ process
 
 
@@ -495,7 +497,7 @@ def for_stmt():
     """
     def process(p):
         (((((((_, var), _), cond), _), action), _), body), _ = p
-        return avo_ast.ForStmt(var, cond, action, body)
+        return statements.ForStmt(var, cond, action, body)
     return (
             keyword('for') + Lazy(assign_stmt) + keyword(';') +
             Exp(b_expr(), None) + keyword(';') +
@@ -511,7 +513,7 @@ def foreach_stmt():
     """
     def process(p):
         (((((_, var), _), val), _), body), _ = p
-        return avo_ast.ForStmt(var, val, body, None)
+        return statements.ForStmt(var, val, body, None)
     return (
             keyword('for') + id_tag + operator('in') + expression() +
             keyword('{') + Opt(Lazy(stmt_list)) + keyword('}')
@@ -532,7 +534,7 @@ def import_stmt():
             objects = [module] + objects
             (_, module), _ = x
             from_import = True
-        return avo_ast.ImportStmt(module, objects, from_import)
+        return statements.ImportStmt(module, objects, from_import)
     return Alt(
         keyword('import') + id_tag + Opt(keyword(',')) + Rep(id_tag + Opt(keyword(','))),
         keyword('from') + id_tag + keyword('import') + id_tag + Opt(keyword(',')) + Rep(id_tag + Opt(keyword(',')))
@@ -552,11 +554,11 @@ def switch_case_stmt():
         cases_list = []
         for c in cases:
             (((_, cond), _), body), _ = c.value
-            cases_list.append(avo_ast.CaseStmt(cond, body))
+            cases_list.append(statements.CaseStmt(cond, body))
         if else_body:
             ((_, _), else_body), _ = else_body
-            cases_list.append(avo_ast.CaseStmt(None, else_body))
-        return avo_ast.SwitchCaseStmt(var, cases_list)
+            cases_list.append(statements.CaseStmt(None, else_body))
+        return statements.SwitchCaseStmt(var, cases_list)
     return (
             keyword('switch') + expression() + keyword('{') +
             Rep(
@@ -582,7 +584,7 @@ def assign_class_stmt():
             interfaces = [i.value for i in interfaces] + [interface]
         else:
             interfaces = []
-        return avo_ast.AssignClassStmt(name, body, inherit, prefix, interfaces)
+        return statements.AssignClassStmt(name, body, inherit, prefix, interfaces)
     return (
             Opt(keyword('abstract')) + keyword('class') + id_tag + Opt(operator(':') + id_tag) +
             Opt(keyword('of') + id_tag + Rep(id_tag)) +
@@ -598,7 +600,7 @@ def assign_interface_stmt():
     """
     def process(p):
         (((_, name), _), body), _ = p
-        return avo_ast.InterfaceStmt(name, body)
+        return statements.InterfaceStmt(name, body)
     return (
             keyword('interface') + id_tag + keyword('{') + Opt(Lazy(interface_body)) + keyword('}')
     ) ^ process
@@ -607,7 +609,7 @@ def assign_interface_stmt():
 def class_body():
     """Class body"""
     def process(p):
-        return avo_ast.StmtList(p)
+        return statements.StmtList(p)
     return Rep(
         Lazy(class_body_stmt) + Opt(keyword(';')) ^ (lambda x: x[0])
     ) ^ process
@@ -616,7 +618,7 @@ def class_body():
 def interface_body():
     """Interface body"""
     def process(p):
-        return avo_ast.StmtList(p)
+        return statements.StmtList(p)
     return Rep(
         Lazy(interface_body_stmt) + Opt(keyword(';')) ^ (lambda x: x[0])
     ) ^ process
@@ -635,10 +637,10 @@ def init_class_stmt():
             (_, args), _ = args
             for arg in args:
                 if arg.value[0][1] is None:
-                    arguments.append(avo_ast.ArgumentAST(arg.value[0][0], None))
+                    arguments.append(expressions.ArgumentAST(arg.value[0][0], None))
                 else:
-                    arguments.append(avo_ast.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
-        return avo_ast.InitClassStmt(arguments, body)
+                    arguments.append(expressions.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
+        return statements.InitClassStmt(arguments, body)
     return (
             keyword('init') + Opt(keyword('(') + Rep(
                id_tag + Opt(operator('=') + Lazy(expression)) + Opt(keyword(','))
@@ -675,7 +677,7 @@ def try_catch_stmt():
     """
     def process(p):
         ((((((_, try_body), _), _), e_name), _), catch_body), _ = p
-        return avo_ast.TryCatchStmt(try_body, e_name, catch_body)
+        return statements.TryCatchStmt(try_body, e_name, catch_body)
     return (
             keyword('try') + keyword('{') + Opt(Lazy(stmt_list)) + keyword('}') +
             keyword('catch') + id_tag + keyword('{') + Opt(Lazy(stmt_list)) + keyword('}')
@@ -704,7 +706,7 @@ def stmt():
             return_stmt() |
             import_stmt() |
             expression() |
-            (Tag(TokenType.EOF) ^ (lambda x: avo_ast.EOFStmt()))
+            (Tag(TokenType.EOF) ^ (lambda x: statements.EOFStmt()))
     )
 
 
